import type IPlayer from "../api/types/IPlayer.ts";
import {Autocomplete, DialogContent, DialogTitle, Drawer, ModalClose, Stack} from "@mui/joy";
import {type Dispatch, useContext, useEffect, useState} from "react";
import {ApiUriContext} from "../api/fetchApi.tsx";
import {LineChart, type LineSeriesType} from "@mui/x-charts";
import PlayerCard from "./PlayerCard.tsx";
import {GamesContext} from "../api/contexts/GamesContext.tsx";
import {GetTimelines} from "../api/endpoints/TimeTrack.tsx";
import type {DatasetElementType} from "@mui/x-charts/internals";

export function PlayerStatsDrawer({player, open, setOpen}: {
    player: IPlayer | null,
    open: boolean,
    setOpen: Dispatch<boolean>
}) {

    const apiUri = useContext(ApiUriContext);
    const games = useContext(GamesContext);

    const [trackedTimes, setTrackedTimes] = useState<DatasetElementType<string | number | Date | null | undefined>[]>([]);
    const [availableSeries, setAvailableSeries] = useState<LineSeriesType[]>([]);
    const [selectedSeries, setSelectedSeries] = useState<LineSeriesType[]>([]);

    useEffect(() => {
        if (!open || !player)
            return;
        GetTimelines(apiUri, player.steamId)
            .then((tt) => {
                const times: DatasetElementType<string | number | Date | null | undefined>[] = [];
                const g: Set<bigint> = new Set<bigint>();
                tt.forEach((app) => {
                    app.value.forEach((time) => {
                        let o = {date: new Date(time.timeStamp)};
                        // @ts-ignore
                        o[`${app.key.toString()}`] = Number(time.timePlayed / 60)
                        times.push(o);
                        g.add(app.key);
                    });
                });

                const seriesTypes: LineSeriesType[] = [];
                const iterator = g.keys();
                let value = iterator.next();
                while (!value.done) {
                    const appId = value.value;
                    seriesTypes.push({
                        type: "line",
                        dataKey: appId.toString(),
                        connectNulls: true,
                        label: games.games.find(g => g.appId == appId)?.name ?? appId?.toString() ?? ""
                    })
                    value = iterator.next();
                }

                setAvailableSeries(seriesTypes);
                setTrackedTimes(times);
            });
    }, [open]);

    return (
        <Drawer anchor={"bottom"} size={"lg"} open={open} onClose={() => setOpen(false)}>
            <ModalClose/>
            <DialogTitle>
                <PlayerCard player={player}/>
            </DialogTitle>
            <DialogContent>
                <Stack direction={"column"} height={"100%"}>
                    <Autocomplete multiple
                                  options={availableSeries.sort((a, b) => (a.label as string).localeCompare(b.label as string))}
                                  placeholder={"Games"}
                                  getOptionLabel={(opt) => opt.label as string}
                                  isOptionEqualToValue={(a, b) => a.dataKey == b.dataKey}
                                  onChange={(_, value, reason) => {
                                      if(reason == "selectOption" || reason == "removeOption" || reason == "clear")
                                          setSelectedSeries(value);
                                      console.log(value);
                                  }}
                    />
                    <LineChart dataset={trackedTimes}
                               xAxis={[{dataKey: "date", scaleType: "utc"}]}
                               yAxis={[{dataKey: "time", scaleType: "linear", min: 0}]}
                               series={selectedSeries}
                    />
                </Stack>
            </DialogContent>
        </Drawer>
    );
}